package cribbage;

import cribbage.scoring.FifteenScorer;
import cribbage.scoring.FlushScorer;
import cribbage.scoring.PairScorer;
import cribbage.scoring.RunScorer;
import cribbage.scoring.Scorer;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class Cribbage {
    private Deck deck;
    List<Scorer> scorers;
    private Hand player;
    private Hand computer;
    private Hand crib;
    private Card starter;
    private int playerCount;
    private int computerCount;
    private boolean playerTurn;
    private boolean gameOver = false;
    
    public Cribbage() {
        deck = new Deck();
        scorers = new ArrayList<Scorer>();
        scorers.add(new FifteenScorer());
        scorers.add(new PairScorer());
        scorers.add(new RunScorer());
        playerTurn = false;
        gameOver = false;
        playerCount = 0;
        computerCount = 0;
    }
    
    public boolean isGameOver() { return gameOver; }
    
    public void deal() {
        if(playerTurn)
            System.out.println("Your crib");
        else
            System.out.println("My crib");
        
        System.out.println("Your hand:");
        Card[] h = new Card[6];
        Card[] c = new Card[4];
        
        for(int n = 0;n < 6;n++) {
            h[n] = deck.deal();
        }
        player = new Hand(h);
        player.print();
        
        System.out.print("Pick two to discard: ");
        int one,two;
        Scanner input = new Scanner(System.in);
        one = input.nextInt();
        c[0] = player.remove(one);
        two = input.nextInt();
        c[1] = player.remove(two);
        player.consolidate();
        
        for(int n = 0;n < 6;n++) {
            h[n] = deck.deal();
        }
        computer = new Hand(h);
        
        int bestScore = 0;
        List<Card> bestSubset = null;
        for(int n = 0;n < 64;n++) {
            List<Card> temp = computer.subset(n);
            if(temp.size() == 4) {
                Hand tempHand = new Hand(temp);
                int points = 0;
                for(int k = 0;k < 16;k++) {
                    List<Card> subset = tempHand.subset(k);
                    for(Scorer sc : scorers) {
                        points += sc.score(subset);
                    }
                }
                if(points > bestScore) {
                    bestScore = points;
                    bestSubset = temp;
                }
            }
        }
        if(bestSubset == null) {
            c[2] = computer.remove(0);
            c[3] = computer.remove(1);
            computer.consolidate();
        } else {
            List<Card> discards = computer.retain(bestSubset);
            c[2] = discards.get(0);
            c[3] = discards.get(1);
        }
        crib = new Hand(c);
        
        starter = deck.deal();
        System.out.println("Starter is " + starter);
        
        if(starter.getFace() == Card.Face.JACK) {
            if(playerTurn)
                playerCount += 2;
            else
                computerCount += 2;
            System.out.println("Dealer scores 2 points for his heels.");
        }
    }
    
    public int scoreHand(Hand h) {
        int points = 0;
        for(int n = 1;n < 32;n++) {
            List<Card> s = h.subset(n);
            for(Scorer sc : scorers) {
                int scored = sc.score(s);
                points += scored;
                if(scored != 0) {
                    System.out.println();
                    System.out.println(sc.what() + ": " + scored);
                    for(Card c : s)
                        System.out.println(c);
                }
            }
        }
        FlushScorer f = new FlushScorer();
        int fs = f.score(h.allCards());
        if(fs > 0) {
            points += fs;
            System.out.println("Hand is flush: " + fs);
        }
        
        if(h.hisNob()) {
            System.out.println("His nob : 1");
            points += 1;
        }

        System.out.println();
        System.out.println("Total points: " + points);
        return points;
    }
    
    public void doRound() {
        if(playerTurn) {
            System.out.println("My hand:");
            computer.addTemp(starter);
            computer.print();
            computerCount += scoreHand(computer);
            computer.removeTemp();
            if(computerCount > 60) {
                System.out.println("My score is now " + computerCount);
                System.out.println("I win!");
                gameOver = true;
                return;
            }
            System.out.println("Your hand:");
            player.addTemp(starter);
            player.print();
            playerCount += scoreHand(player);
            player.removeTemp();
            System.out.println("Your crib:");
            crib.addTemp(starter);
            crib.print();
            playerCount += scoreHand(crib);
            crib.removeTemp();
            if(playerCount > 60) {
                System.out.println("Your score is now " + playerCount);
                System.out.println("You win!");
                gameOver = true;
            }
        } else {
            System.out.println("Your hand:");
            player.addTemp(starter);
            player.print();
            playerCount += scoreHand(player);
            player.removeTemp();
            if(playerCount > 60) {
                System.out.println("Your score is now " + playerCount);
                System.out.println("You win!");
                gameOver = true;
                return;
            }
            System.out.println("My hand:");
            computer.addTemp(starter);
            computer.print();
            computerCount += scoreHand(computer);
            computer.removeTemp();
            System.out.println("My crib:");
            crib.addTemp(starter);
            crib.print();
            computerCount += scoreHand(crib);
            crib.removeTemp();
            if(computerCount > 60) {
                System.out.println("I win!");
                gameOver = true;
            }
        }
        System.out.println("My score is now " + computerCount);
        System.out.println("Your score is now " + playerCount);
                     
        playerTurn = !playerTurn;
        deck.shuffle();
    }
    
    public static void main(String[] args) {
        Cribbage game = new Cribbage();
        while(!game.isGameOver()) {
            game.deal();
            game.doRound();
        }
    }
    
}
