package cribbage;

import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedList;

public class Hand {
    private List<Card> cards;
    private Card temp;
    
    public Hand(Card[] cs) {
        Arrays.sort(cs);
        cards = new ArrayList<Card>();
        for(Card c : cs)
            cards.add(c);
    }
    
    public Hand(List<Card> cs) {
        Collections.sort(cs);
        cards = cs;
    }
    
    public void print() {
        for(int n = 0; n < cards.size();n++)
            System.out.println(String.valueOf(n) + ") " + cards.get(n));
    }
    
    public List<Card> allCards() { return cards; }
    
    public void addTemp(Card c) {
        temp = c;
        cards.add(c);
        Collections.sort(cards);
    }
    
    public void removeTemp() {
        cards.remove(temp);
        temp = null;
    }
    
    public Card remove(int n) {
       Card discard = cards.get(n);
       cards.set(n, null);
       return discard;
    }
    
    /** Calling remove() sets some of the cards in the hand to null.
     *  To avoid problems caused by null cards, call consolidate()
     *  after calling remove. consolidate() removes the null cards.
     */
    public void consolidate() {
        ArrayList<Card> newCards = new ArrayList<Card>();
        
        for(Card c : cards) {
            if(c != null)
                newCards.add(c);    
        }
        cards = newCards;
    }
    
    public List<Card> subset(int n) {
        LinkedList<Card> s = new LinkedList<Card>();
        int loc = this.cards.size() - 1;
        while(n > 0 && loc >= 0) {
            if(n % 2 == 1)
                s.addFirst(cards.get(loc));
            n = n/2;
            loc--;
        }
        return s;
    }
    
    /** Given a subset of cards to keep, remove and return unwanted cards. **/
    public List<Card> retain(List<Card> keep) {
        ArrayList<Card> discards = new ArrayList<Card>();
        for(Card c : cards)
            if(!keep.contains(c))
                discards.add(c);
        this.cards = keep;
        return discards;
    }
    
    public boolean hisNob() {
        // Check to see if the hand contains a Jack of the same suit as the
        // starter card
        for(Card c : cards)
            if(c.getFace() == Card.Face.JACK && c.getSuit() == temp.getSuit() && c != temp)
                return true;
        return false;
    }
    
    public static void main(String[] args) {
        Deck d = new Deck();
        Card[] h = new Card[6];
        for(int n = 0;n < 6;n++) {
            h[n] = d.deal();
        }
        Hand hand = new Hand(h);
        System.out.println("Hand");
        hand.print();
        
        for(int n = 1;n < 64;n++) {
            List<Card> s = hand.subset(n);
            int pts = 0;
            for(Card c : s) {
                pts += c.getPoints();
            }
            if(pts == 15) {
                System.out.println();
                System.out.println("Subset adds to 15:");
                for(Card c : s)
                    System.out.println(c);
            }
        }
    }
}
